﻿properties { 
 
  $version = '2.4.2'  
  $zipFileName = "AzureToolkit_V$version.zip"
  $buildDocumentation = $false
  $buildNuPackage = $true
  
  $baseDir  = resolve-path ..
  $buildDir = "$baseDir\Build"
  $sourceDir = "$baseDir\Source"
  $toolsDir = "$baseDir\Tools"
  $docDir = "$baseDir\Doc"
  $releaseDir = "$baseDir\Release"
  $workingDir = "$baseDir\Working"
  $builds = @(
    @{Name = "AzureToolkit"; TestsName = $null; Constants=""; FinalDir="Net40"; Framework="net-4.0"}
  )
}

$framework = '4.0x86'

task default -depends Test

# Ensure a clean working directory
task Clean {
  Set-Location $baseDir
  
  if (Test-Path -path $workingDir)
  {
    Write-Output "Deleting Working Directory"
    
    del $workingDir -Recurse -Force
  }
  
  New-Item -Path $workingDir -ItemType Directory
}

# Build each solution, optionally signed
task Build -depends Clean { 
  foreach ($build in $builds)
  {
    $name = $build.Name
    $finalDir = $build.FinalDir

    Write-Host -ForegroundColor Green "Building " $name
    Write-Host
    exec { msbuild "/t:Clean;Rebuild" /p:Configuration=Release /p:OutputPath=bin\Release\$finalDir\ (GetConstants $build.Constants $signAssemblies) ".\Source\$name.sln" } "Error building $name"
  }
}

# Create Merged Versions of the builds that use the DLR
 task Merge -depends Build {
  #$binaryDir = "$sourceDir\Facebook\bin\Release\DotNet35"  
  #MergeAssembly "$binaryDir\Facebook.dll" "v2" $false $signKeyPath @("""$binaryDir\Microsoft.Dynamic.dll""", """$binaryDir\Microsoft.Scripting.dll""", """$binaryDir\Microsoft.Scripting.Core.dll""", """$binaryDir\Microsoft.Scripting.ExtensionAttribute.dll""")

  #$binaryDir = "$sourceDir\Newtonsoft.Json.Tests\bin\Release\DotNet20"
  #MergeAssembly "$binaryDir\Newtonsoft.Json.Tests.Net20.dll" $signKeyPath "$binaryDir\LinqBridge.dll"
  #MergeAssembly "$binaryDir\Newtonsoft.Json.Net20.dll" $signKeyPath "$binaryDir\LinqBridge.dll"
  #del $binaryDir\LinqBridge.dll
 }

# Optional build documentation, add files to final zip
task Package -depends Merge {
  foreach ($build in $builds)
  {
    $name = $build.TestsName
    $finalDir = $build.FinalDir
    
    robocopy $sourceDir\AzureToolkit\bin\Release\$finalDir $workingDir\Package\Bin\$finalDir /S /NP /XF *.sdf *.old
  }
  
  if ($buildDocumentation)
  {
    exec { msbuild "/t:Clean;Rebuild" /p:Configuration=Release "/p:DocumentationSourcePath=$workingDir\Package\Bin\Net40" $docDir\doc.shfbproj } "Error building documentation. Check that you have Sandcastle, Sandcastle Help File Builder and HTML Help Workshop installed."
    
    move -Path $workingDir\Documentation\Documentation.chm -Destination $workingDir\Package\Documentation.chm
    move -Path $workingDir\Documentation\LastBuild.log -Destination $workingDir\Documentation.log
  }
  if ($buildNuPackage)
  {
    New-Item -Path $workingDir\NuGet -ItemType Directory
    New-Item -Path $workingDir\NuGet\AzureToolkit\$version\ -ItemType Directory
    Copy-Item -Path "$buildDir\AzureToolkit.nuspec" -Destination $workingDir\NuGet\AzureToolkit\$version\AzureToolkit.nuspec -recurse
    
    (Get-Content $workingDir\NuGet\AzureToolkit\$version\AzureToolkit.nuspec) | 
    Foreach-Object {$_ -replace "{version}", $version} | 
    Set-Content $workingDir\NuGet\AzureToolkit\$version\AzureToolkit.nuspec
    
    foreach ($build in $builds)
    {
        $name = $build.TestsName
        $finalDir = $build.FinalDir

        New-Item -Path $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir -ItemType Directory
        New-Item -Path $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir\CodeContracts -ItemType Directory
        Copy-Item -Path "$baseDir\Source\AzureToolkit\Bin\Release\$finalDir\AzureToolkit.dll" -Destination $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir
        Copy-Item -Path "$baseDir\Source\AzureToolkit\Bin\Release\$finalDir\AzureToolkit.pdb" -Destination $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir
        Copy-Item -Path "$baseDir\Source\AzureToolkit\Bin\Release\$finalDir\AzureToolkit.XML" -Destination $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir
        Copy-Item -Path "$baseDir\Source\AzureToolkit\Bin\Release\$finalDir\CodeContracts\AzureToolkit.Contracts.dll" -Destination $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir\CodeContracts
        Copy-Item -Path "$baseDir\Source\AzureToolkit\Bin\Release\$finalDir\CodeContracts\AzureToolkit.Contracts.pdb" -Destination $workingDir\NuGet\AzureToolkit\$version\lib\$finalDir\CodeContracts   
    }
  
    exec { .\Tools\NuGet\NuGet.exe pack $workingDir\NuGet\AzureToolkit\$version\AzureToolkit.nuspec -o $workingDir\NuGet\ }
    move -Path .\*.nupkg -Destination $workingDir\NuGet
  }
  
  Copy-Item -Path $docDir\readme.txt -Destination $workingDir\Package\
  #Copy-Item -Path $docDir\versions.txt -Destination $workingDir\Package\Bin\

  robocopy $sourceDir $workingDir\Package\Source\Source /MIR /NP /XD .svn bin obj TestResults /XF *.suo *.user
  robocopy $buildDir $workingDir\Package\Source\Build /MIR /NP /XD .svn
  robocopy $docDir $workingDir\Package\Source\Doc /MIR /NP /XD .svn
  robocopy $toolsDir $workingDir\Package\Source\Tools /MIR /NP /XD .svn
  
  exec { .\Tools\7-zip\7za.exe a -tzip $workingDir\$zipFileName $workingDir\Package\* } "Error zipping"
}

# Run tests on deployed files
task Test -depends Package {
  #foreach ($build in $builds)
  #{
  #  $name = $build.TestsName
  #  if ($name -ne $null)
  #  {
  #      $finalDir = $build.FinalDir
  #      $framework = $build.Framework
  #      
  #      Write-Host -ForegroundColor Green "Copying test assembly $name to deployed directory"
  #      Write-Host
  #      robocopy ".\Src\Newtonsoft.Json.Tests\bin\Release\$finalDir" $workingDir\Deployed\Bin\$finalDir /NP /XO /XF LinqBridge.dll
  #      
  #      Copy-Item -Path ".\Src\Newtonsoft.Json.Tests\bin\Release\$finalDir\$name.dll" -Destination $workingDir\Deployed\Bin\$finalDir\

  #      Write-Host -ForegroundColor Green "Running tests " $name
  #      Write-Host
  #      exec { .\Tools\NUnit\nunit-console.exe "$workingDir\Deployed\Bin\$finalDir\$name.dll" /framework=$framework /xml:$workingDir\$name.xml } "Error running $name tests"
  #  }
  #}
}

function MergeAssembly($dllPrimaryAssembly, $targetPlatform, $internalize, $signKey, [string[]]$mergedAssemlies)
{
  $mergeAssemblyPaths = [String]::Join(" ", $mergedAssemlies)
  
  $primary = Get-Item $dllPrimaryAssembly
  $mergedAssemblyName = $primary.Name
  $mergedDir = $primary.DirectoryName + "\Merged"
  
  Remove-Item $mergedDir -recurse
  New-Item $mergedDir -ItemType Directory
  
  $ilMergeKeyFile = switch($signAssemblies) { $true { "/keyfile:$signKeyPath" } default { "" } }
  $internalizeSwitch = switch($internalize) { $true { "/internalize" } default { "" } }
  
  exec { .\Tools\ILMerge\ilmerge.exe "/targetplatform:$targetPlatform" $internalizeSwitch "/closed" "/log:$workingDir\$mergedAssemblyName.MergeLog.txt" $ilMergeKeyFile "/out:$mergedDir\$mergedAssemblyName" $dllPrimaryAssembly $mergeAssemblyPaths } "Error executing ILMerge"
}

function GetConstants($constants, $includeSigned)
{
  $signed = switch($includeSigned) { $true { ";SIGNED" } default { "" } }

  return "/p:DefineConstants=`"TRACE;$constants$signed`""
}